@@ -15,9 +15,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -1212,6 +1210,9 @@ static int dsp_open(int flags, int *_errno) {
         i->io_flags = PA_IO_EVENT_INPUT | PA_IO_EVENT_OUTPUT;
         break;
     default:
+        pa_threaded_mainloop_unlock(i->mainloop);
+        fd_info_unref(i);
+        *_errno = EIO;
         return -1;
     }
 
@@ -1910,7 +1911,7 @@ static int dsp_trigger(fd_info *i) {
     }
 
     i->operation_success = 0;
-    while (!pa_operation_get_state(o) != PA_OPERATION_DONE) {
+    while (pa_operation_get_state(o) != PA_OPERATION_DONE) {
         PLAYBACK_STREAM_CHECK_DEAD_GOTO(i, fail);
 
         pa_threaded_mainloop_wait(i->mainloop);
@@ -1945,7 +1946,7 @@ static int dsp_cork(fd_info *i, pa_stream *s, int b) {
     }
 
     i->operation_success = 0;
-    while (!pa_operation_get_state(o) != PA_OPERATION_DONE) {
+    while (pa_operation_get_state(o) != PA_OPERATION_DONE) {
         if (s == i->play_stream)
             PLAYBACK_STREAM_CHECK_DEAD_GOTO(i, fail);
         else if (s == i->rec_stream)
@@ -2733,12 +2734,12 @@ __attribute__((constructor)) void _init (void);
 
 __attribute__((constructor))
 void _init (void) {
-  cygwin_internal (CW_HOOK, "_open64", open);
+  cygwin_internal (CW_HOOK, "open", open);
   cygwin_internal (CW_HOOK, "ioctl", ioctl);
   cygwin_internal (CW_HOOK, "close", close);
   cygwin_internal (CW_HOOK, "access", access);
-  cygwin_internal (CW_HOOK, "_stat64", stat);
-  cygwin_internal (CW_HOOK, "_fopen64", fopen);
+  cygwin_internal (CW_HOOK, "stat", stat);
+  cygwin_internal (CW_HOOK, "fopen", fopen);
   cygwin_internal (CW_HOOK, "fclose", fclose);
 }
 #endif
